@@ -107,9 +107,9 @@ unsigned char *extract_zip(f_state *s, u_int64_t c_offset, unsigned char *founda
 			//Size of the local file header data structure
 			foundat += 30;									
 
-			if (needle->suffix == "zip")
+			if (strcmp(needle->suffix,"zip")==0)
 				{
-				if (strncmp((char *)foundat, "content.xml", 11) == 0 && needle->suffix == "zip")
+				if (strncmp((char *)foundat, "content.xml", 11) == 0 && strcmp(needle->suffix,"zip")==0)
 					{
 					oOffice = TRUE;
 					sprintf(comment, " (OpenOffice Doc?)");
@@ -138,7 +138,7 @@ unsigned char *extract_zip(f_state *s, u_int64_t c_offset, unsigned char *founda
 			if(localFH.genFlag & 1<<3 && localFH.uncompressed==0 &&  localFH.compressed==0 )
 				{
 #ifdef DEBUG
-				fprintf(stderr,"No data to jmp Just search for the next file Footer\n",localFH.genFlag);
+				fprintf(stderr,"No data to jmp Just search for the next file Footer (localFH.genFlag:=%d)\n",localFH.genFlag);
 #endif
 				break;
 				}
@@ -175,14 +175,21 @@ unsigned char *extract_zip(f_state *s, u_int64_t c_offset, unsigned char *founda
 		}
 	else
 		{
-		bytes_to_search = (foundat - buf);
+		bytes_to_search = (buflen < (foundat - buf) ? buflen : buflen - (foundat - buf));
 		}
 
 	//Make sure we are not searching more than what he have
-	if (buflen - (foundat - buf) < bytes_to_search)
+        if (buflen <= (foundat - buf)) {
+#ifdef DEBUG
+		printf("avoided bug in extract_zip!\n");
+#endif
+		bytes_to_search = 0;
+	} else {
+		if (buflen - (foundat - buf) < bytes_to_search)
 		{
 		bytes_to_search = buflen - (foundat - buf);
 		}
+	}
 
 
 	currentpos = foundat;
@@ -206,7 +213,7 @@ unsigned char *extract_zip(f_state *s, u_int64_t c_offset, unsigned char *founda
 
 		/*Jump to the comment length field*/
 #ifdef DEBUG
-		printf("distance searched:=%d \n", foundat - currentpos);
+		printf("distance searched:=%lu\n", foundat - currentpos);
 #endif
 		if (buflen - (foundat - buf) > 20)
 			{
@@ -225,7 +232,7 @@ unsigned char *extract_zip(f_state *s, u_int64_t c_offset, unsigned char *founda
 		printf("Found a %s type:=%s\n", needle->suffix, type);
 #endif
 		extractbuf = buf;
-		if (type == "all" || type == needle->suffix)
+		if (strcmp(type,"all")==0 || strcmp(type,needle->suffix)==0)
 		{
 #ifdef DEBUG
 			printf("Writing a %s to disk\n", needle->suffix);
@@ -963,7 +970,7 @@ unsigned char *extract_ole(f_state *s, u_int64_t c_offset, unsigned char *founda
 	if (suffix)
 		needle->suffix = suffix;
 
-	if (!strstr(needle->suffix, type) && type != "all")
+	if (!strstr(needle->suffix, type) && strcmp(type,"all")!=0)
 		{
 		return foundat;
 		}
@@ -1094,7 +1101,7 @@ unsigned char *extract_mov(f_state *s, u_int64_t c_offset, unsigned char *founda
 		printf("file size:=%d\n", filesize);
 #endif
 		extractbuf = buf;
-		write_to_disk(s, needle, filesize, extractbuf, c_offset + f_offset);
+		write_to_disk(s, needle, filesize, extractbuf, c_offset + f_offset - 4);
 		return foundat;
 		}
 
@@ -1235,7 +1242,7 @@ unsigned char *extract_riff(f_state *s, u_int64_t c_offset, unsigned char *found
 				file_size = size;
 				extractbuf = buf;
 				needle->suffix = "avi";
-				if (!strstr(needle->suffix, type) && type != "all")
+				if (!strstr(needle->suffix, type) && strcmp(type,"all")!=0)
 					return foundat + size;
 				write_to_disk(s, needle, file_size, extractbuf, c_offset + f_offset);
 				foundat += size;
@@ -1261,7 +1268,7 @@ unsigned char *extract_riff(f_state *s, u_int64_t c_offset, unsigned char *found
 			file_size = size;
 			extractbuf = buf;
 			needle->suffix = "wav";
-			if (!strstr(needle->suffix, type) && type != "all")
+			if (!strstr(needle->suffix, type) && strcmp(type,"all")!=0)
 				return foundat + size;
 
 			write_to_disk(s, needle, file_size, extractbuf, c_offset + f_offset);
@@ -1350,7 +1357,7 @@ unsigned char *extract_bmp(f_state *s, u_int64_t c_offset, unsigned char *founda
 		file_size = size;
 		extractbuf = buf;
 		
-		write_to_disk(s, needle, file_size, extractbuf, (c_offset + f_offset)-4);
+		write_to_disk(s, needle, file_size, extractbuf, (c_offset + f_offset));
 		foundat += file_size;
 		return foundat;
 
@@ -1660,7 +1667,7 @@ unsigned char *extract_png(f_state *s, u_int64_t c_offset, unsigned char *founda
 		if (size <= 0 || size > buflen - (foundat - buf))
 		{
 #ifdef DEBUG
-			printf("found-buf=%d\n", buflen - (foundat - buf));
+			printf("buflen - (foundat-buf)=%lu\n", buflen - (foundat - buf));
 #endif
 			return currentpos;
 		}
@@ -1718,7 +1725,7 @@ unsigned char *extract_jpeg(f_state *s, u_int64_t c_offset, unsigned char *found
 	unsigned char	*currentpos = NULL;
 
 	unsigned char	*extractbuf = NULL;
-	unsigned short	headersize = 0;
+	unsigned short	headersize;
 	int				bytes_to_search = 0;
 	int				hasTable = FALSE;
 	int				hasHuffman = FALSE;
@@ -1796,10 +1803,17 @@ unsigned char *extract_jpeg(f_state *s, u_int64_t c_offset, unsigned char *found
 	currentpos = foundat;
 
 	//sprintf("Searching for footer\n");
-	if (buflen - (foundat - buf) >= needle->max_len)
-		bytes_to_search = needle->max_len;
-	else
-		bytes_to_search = buflen - (foundat - buf);
+	if (buflen < (foundat - buf)) {
+#ifdef DEBUG
+		printf("avoided bug in extract_jpeg!\n");
+#endif
+		bytes_to_search = 0;
+	} else {
+		if (buflen - (foundat - buf) >= needle->max_len)
+			bytes_to_search = needle->max_len;
+		else
+			bytes_to_search = buflen - (foundat - buf);
+	}
 
 	foundat = bm_search(needle->footer,
 						needle->footer_len,
@@ -1911,7 +1925,7 @@ unsigned char *extract_generic(f_state *s, u_int64_t c_offset, unsigned char *fo
 	else
 	{
 #ifdef DEBUG
-		printf("footer is not NULL %d\n", needle->footer);
+		printf("footer is not NULL %p\n", needle->footer);
 #endif
 		foundat = bm_search(needle->footer,
 							needle->footer_len,
